==> I2CE-4.0.22.tar.bz2: I2CE/modules/Forms/lib/I2CE_PageForm.php <==
<?php
/**
 * @copyright © 2007, 2008, 2009 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @package I2CE
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @since v1.0.0
 * @version v2.0.0
 */
/**
 * Handles all page display and operations for forms.
 *  
 * This class extends the {@link I2CE_Page} class and adds some default methods and variables
 * for form templates that will interface with a {@link I2CE_Form} object to update the
 * database.  The {@link loadObjects()} method must be overwritten for all objects
 * extending this abstract class.
 * @package I2CE
 * @abstract
 * @access public
 * @see I2CE_Template
 */
abstract class I2CE_PageForm extends I2CE_Page {

    /**
     * Constant values for the type of object being set to be edited for this page: Primary
     * The primary object is the main object being edited by this page.  There can only be one.
     */
    const EDIT_PRIMARY = 1;
    /**
     * Constant values for the type of object being set to be edited for this page: Parent
     * The parent for the primary and secondary objects being edited by this page.  The can only be one.
     */
    const EDIT_PARENT = 2;
    /**
     * Constant values for the type of object being set to be edited for this page: Secondary
     * Any secondary objects being edited by this page.  They share the parent of the primary.  Multiple objects
     * can be added.
     */
    const EDIT_SECONDARY = 3;
    /**
     * Constant values for the type of object being set to be edited for this page: Children
     * Any child objects of the primary object will use this type.  Multiple objects can be added.
     */
    const EDIT_CHILD = 4;
        
    /**
     * The objects related to this page.  Using the PAGE_FORM_EDIT constants to determine the key for the 
     * array and the values will either be a {@link I2CE_Form} or an array of {@link I2CE_Form}s.
     * @var array
     */
    protected $objects;
    /**
     * The node id's of objects related to this page.
     * @var array
     */
    protected $node_ids;
    /**
     * A flag to determine if the page is being edited for the dynamic lists.
     * @var boolean
     */
    private $editing;
    /**
     * The factory object that creates new instances of forms
     * @var I2CE_FormFactory
     */
    protected $factory;
        
    /**
     * An array of the type of button templates to be used for thie page
     * @var protected array $button_templates
     */
    protected $button_templates;
        
    /**
     * Create a new instance of a form page.
     * 
     * This will call the constructor for all Page objects and then set up some additional
     * member variables for forms.
     * @param string $title The title for this page.
     * @param string $defaultHTMLFile The default HTML file for this page.
     * @param mixed $access The role required to access this page.
     * @param array $files The list of template files to load for this page.
     */
    public function __construct( $args,$request_remainder, $get = null,$post = null) {
        parent::__construct( $args,$request_remainder,$get,$post);
        $this->template->addHeaderLink('mootools-core.js');
        $this->template->addHeaderLink('I2CE_ClassValues.js');
        $this->template->addHeaderLink('I2CE_SubmitButton.js');
        $this->button_templates = array(
            'button_save'=> 'button_save.html',
            'button_save_only'=> 'button_save_only.html',
            'button_save_return'=>'button_save_return.html',
            'button_confirm'=> 'button_confirm.html' ,
            'button_return_only'=> 'button_return_only.html'
            );
        if (array_key_exists('buttons',$this->args) && is_array($this->args['buttons']))  {
            foreach (array_keys($this->button_templates) as $key) {
                if (array_key_exists($key,$this->args['buttons']) && is_string($this->args['buttons'][$key]) && strlen($this->args['buttons'][$key]) > 0) {
                    $this->button_templates[$key] = $this->args['buttons'][$key];
                }
            }
        }
        $this->editing = false;
        $this->template->setShowForm();
        $this->node_ids = array();
        $this->objects = array( 
            self::EDIT_PRIMARY => false,
            self::EDIT_PARENT => false,
            self::EDIT_SECONDARY => array(),
            self::EDIT_CHILD => array(),
            );
        $this->factory = I2CE_FormFactory::instance();
        if (array_key_exists('confirm', $args)) {
            $this->usesConfirmPage = $args['confirm'];
        } else {
            $this->usesConfirmPage = true;
        }
        $this->formAttributes = null;
    }
        
    /**
     * Set this page to be an editing page for the dynamic lists.
     */
    protected final function setEditing() {
        $this->editing = true;
    }
    /**
     * Check to see if this page is an editing page and already has data populated.
     * @return boolean;
     */
    protected final function isEditing() {
        return $this->editing || $this->isPost();
    }
        
    /**
     * Add an object to the objects being edited or displayed by this page.
     * @param I2CE_Form $object  
     * @param integer $type The type of object being added from EDIT constants.
     * @param mixed $node_id  a tring which gives the node id for which this object applies to. also can be the DOMNode.   
     * Defaults to null meaning that it applies to the whole page
     *
     */
    protected final function setObject( $object, $type = self::EDIT_PRIMARY,$node_id = null ) {
        switch ( $type ) {
        case self::EDIT_PARENT :
        case self::EDIT_PRIMARY :
            $this->objects[$type] = $object;
            $this->node_ids[$type] = $node_id;
            break;
        case self::EDIT_SECONDARY :
        case self::EDIT_CHILD :
            $this->objects[$type][] = $object;
            $this->node_ids[$type][] = $node_id;
            break;
        }
    }
        
    /**
     * Return the primary object being edited.
     * @return I2CE_Form
     */
    protected function getPrimary() {
        return $this->objects[self::EDIT_PRIMARY];
    }
    /**
     * Return the parent object for this form.
     * @return I2CE_Form
     */
    protected function getParent() {
        return $this->objects[self::EDIT_PARENT];
    }
        
    /**
     * Set the I2CE_Form object in the page template.
     * 
     * This method will pass the edit object to the page template so that it can process all the form variables.
     */
    protected function setForm() {
        foreach( $this->objects as $type => $edit ) {
            if ( is_array( $edit ) ) {
                foreach( $edit as $i=> $obj ) {
                    if( $obj instanceof I2CE_Form ) {
                        if ($this->node_ids[$type][$i] !== null) {
                            $this->template->setForm( $obj, $this->node_ids[$type][$i] );
                        } else {
                            $this->template->setForm( $obj );
                        }
                    }
                }
            } elseif ( $edit instanceof I2CE_Form ) {
                if ($this->node_ids[$type] !== null) {
                    $this->template->setForm( $edit, $this->node_ids[$type] );
                } else {
                    $this->template->setForm( $edit );
                }
            }
        }
    }


    /**
     * Set the data to be displayed for the outside of the form field elements.
     * 
     * Set up the static data to be displayed in the template.  The default method
     * doesn't do anything, but sub-classes may need to override this method.
     *  */
    protected function setDisplayData() {
    }


    /**
     * Display the save or confirm buttons as needed.
     * 
     * If the page is a confirmation view then the save / edit button template will be displayed.  
     * Otherwise the confirm and return buttons will be shown.
     * @param boolean $save Flag to show the save button. (Defaults to false)
     * @param boolean $show_edit (defaults to true)
     * @global array
     */
    protected  function displayControls( $save = false, $show_edit = true ) {
        $i2ce_config = I2CE::getConfig()->I2CE;        
        if ( $save ) {
            if ( $show_edit ) {
                $this->template->addFile( $this->button_templates['button_save'] );
            } else {
                if ($this->usesConfirmPage) {
                    $this->template->addFile( $this->button_templates['button_save_only']);
                } else {
                    $this->template->addFile( $this->button_templates['button_save_return'] );
                }
            }
        }  else {
            if ($show_edit) {
                $this->template->addFile( $this->button_templates['button_confirm'] );     
            } else {
                $this->template->addFile( $this->button_templates['button_return_only'] );     
            }
        }
    }

    /**
     * Create and load any necessary objects for this form.
     * 
     * This method must be written for each class extending this class.
     */
    protected function loadObjects() {
        if ( $this->getParent() instanceof I2CE_Form ) {
            if ($this->getPrimary() instanceof I2CE_Form) {
                $this->getPrimary()->setParent( $this->getParent() );
            }
            foreach( $this->objects[self::EDIT_SECONDARY] as $secondary ) {
                $secondary->setParent( $this->getParent() );
            }
        }
        if ( $this->isPost() ) {
            if ($this->getPrimary() instanceof I2CE_Form) {
                $this->getPrimary()->load( $this->post);
            }
            foreach( $this->objects[self::EDIT_CHILD] as $child ) {                
                $child->load( $this->post ,  $child->getId() != '0');                
                $child->setParent( $this->getPrimary(), true );
            }
            foreach( $this->objects[self::EDIT_SECONDARY] as $secondary ) {
                $secondary->load( $this->post ,  $secondary->getId()!= '0');
            }
        } elseif ( $this->getPrimary() instanceof I2CE_Form ) {
            if ( $this->getPrimary()->getId() != '0' ) {
                $this->getPrimary()->populate();
                foreach( $this->objects[self::EDIT_CHILD] as $child ) {
                    $child->setParent( $this->getPrimary(), true );
                    $child->populate();
                }
                // Nothing needs to be done for secondary objects here because 2 objects can't be edited on the same form,
                // only new objects can be added to the database.
                $this->setEditing();
            }
        }
        return true;
    }
        
    /**
     * Run the validation methods for all the objects being edited.
     * 
     * If this is a form submit then run the validation methods for the default object being edited.  The default method
     * calls the {@link I2CE_Form::validate() validate} method on the {@link $edit_obj} object.
     */
    protected function validate() {
        if ( $this->isPost() ) {
            foreach( $this->objects as $type => $edit ) {
                if ( $type == self::EDIT_PARENT ) continue;
                if ( is_array( $edit ) ) {
                    foreach( $edit as $i=>$obj ) {
                        if( $obj instanceof I2CE_Form ) {
                            $obj->validate();
                        }
                    }
                } elseif ( $edit instanceof I2CE_Form ) {
                    $edit->validate();
                }
            }
        }
    }
                
    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited b
     * @global array
     */
    protected function save() {
        $this->getPrimary()->save( $this->user );
        foreach( $this->objects[ self::EDIT_SECONDARY] as $obj ) {
            if (!$obj->save( $this->user )) {
                return false;
            }
        }
        foreach( $this->objects[ self::EDIT_CHILD] as $obj ) {
            if ( $obj->getId() == '0' ) $obj->setParent( $this->getPrimary() );
            if (!$obj->save( $this->user )) {
                return false;
            }
        }
        return true;
    }
        


    /**
     * Perform the actions of the page.
     * 
     * The default method sets up a form object to display the form and confirmation pages
     * for the given object.  It handles everything necessary for editing and saving
     * a single object.  Some forms may need to override this if the actions are more complex.
     * Or more simple.
     */
    protected function action() {
        if ($this->loadObjects() === false) {
            return false;
        }
        // This is happening later so to keep it from being called multiple times
        //$this->validate();
        if ( $this->isSave() ) {
            $this->save();
        } else {
            $this->setForm();
            $this->setDisplayData();
            $this->loadHTMLTemplates();
            if ($this->formAttributes) {
                $html_form = $this->template->getElementByTagName('form',0);
                if (!($html_form)) {
                    I2CE::raiseError("No form found when setting the encoding type",E_USER_WARNING);
                }
                foreach ($this->formAttributes as $attrib=>$value) {
                    $html_form->setAttribute($attrib,$value);
                }
            }         
            if ($this->usesConfirmPage) {
                if ( $this->isConfirm() ) {
                    $this->setDisplayData();
                    $this->template->setReview();
                    $save = true;
                    $show_edit = true;
                } else {
                    if ( $this->isPost() ) {
                        $this->invalidMessage();
                    }
                    $save = false;
                    $show_edit = true;
                }
            } else {
                if ( $this->isPost() ) {
                    $this->invalidMessage();
                }
                $save = true;
                $show_edit = false;
            }
            $this->displayControls($save,$show_edit);
        }

    }
        
    /**
     *Checks to see if any of the forms on this page have invalid messages
     *@returns boolean
     */
    public function hasInvalid() {
        $invalid = false;        
        foreach (array(self::EDIT_PARENT,self::EDIT_PRIMARY) as $key) {
            if ( !$this->objects[$key]  instanceof I2CE_Form) {
                continue;
            }
            $invalid |= $this->objects[$key]->hasInvalid();
        }
        foreach (array(self::EDIT_SECONDARY,self::EDIT_CHILD) as $key) {
            if (!is_array($this->objects[$key])) {
                continue;
            }
            foreach ($this->objects[$key] as $form) {
                if (!$form instanceof I2CE_Form) {
                    continue;
                }
                $invalid |= $form->hasInvalid();
            }
        }
        return $invalid;
    }


    /**
     * Add the form_error template to the page if the template is marked as invalid.
     */
    public  function invalidMessage() {  
        if (! $this->hasInvalid()){
            return;
        }
        $i2ce_config = I2CE::getConfig()->modules->forms;        
        $node = $this->template->addFile( $i2ce_config->template->form_error );
        if (!$node instanceof DOMNode) {
            return;
        }
        foreach (array(self::EDIT_PARENT,self::EDIT_PRIMARY) as $key) {
            if ( !$this->objects[$key]  instanceof I2CE_Form) {
                continue;
            }
            if ( $this->objects[$key]->hasInvalid()) {
                foreach ($this->objects[$key] as $field) {
                    if (!$field instanceof I2CE_FormField ) {
                        continue;
                    }
                    if ($field->hasInvalid()) {
                        $node->appendChild($this->template->createElement('span',array('style'=>'display:none'),$field->getHTMLName()));
                    }
 
                }
            }
        }
        foreach (array(self::EDIT_SECONDARY,self::EDIT_CHILD) as $key) {
            if (!is_array($this->objects[$key])) {
                continue;
            }
            foreach ($this->objects[$key] as $form) {
                if (!$form instanceof I2CE_Form) {
                    continue;
                }
                if ($form->hasInvalid()) {
                    foreach ($form as $field) {
                        if (!$field instanceof I2CE_FormField ) {
                            continue;
                        }
                        if ($field->hasInvalid()) {
                            $node->appendChild($this->template->createElement('span',array('style'=>'display:none'),$field->getHTMLName()));
                        }
                    }
                }
            }
        }
    }



    /**
     * @var protected boolean $usesConfirmPage
     */
    protected $usesConfimPage;
        
    /** Set whether or not we use the confirm page when submitting the form
     * @param boolean $val.  True if we use a confirm page
     */
    public function usesConfirmPage($val) {
        $this->usesConfirmPage = $val;
    }

    /**
     * @var protected array $formAttributes an associative array of attributes to set for the form
     */
    protected $formAttributes;

    /**
     * Set any attributes for the form
     * @param array $formAttributes  an associative array of attributes to set for the form. Keys are
     * attributes, values are values.
     */
    public function setFormAttributes($formAttributes){
        $this->formAttributes = $formAttributes;
    }



    /**
     * Checks to see if the page is being submitted as a save operation.
     * @param boolean $validate Flag to also check to be sure the form data is valid.
     * @return boolean
     */
    protected function isSave( $validate = true ) {
        if ( ! ( $this->isPost() && $this->post_exists('submit_type') && $this->post( 'submit_type' ) == "save" )) {
            return false;
        }
        if ($validate) {
            //we are trying to save.. need to verify that all is validated
            $this->validate();
            if ($this->hasInvalid()) {
                return false;
            }
        }
        return true;
    }
    /**
     * Checks to see if the page is being submitted as an edit operation.
     * @param boolean $validate Flag to also check to be sure the form data is valid.
     * @return boolean
     */
    protected function isEdit( $validate = true ) {
        if ( $this->post( 'submit_type' ) == "edit" ) {
            if ( $validate ? !$this->hasInvalid() : true ) {
                return true;
            } 
        }
        return false;
    }
    
    /**
     * Checks to see if the page is a confirmation page.
     * @return boolean
     */
    protected function isConfirm( $validate = true ) {
        if ( ! ( $this->isPost() && $this->post_exists('submit_type') && $this->post( 'submit_type' ) == "confirm" )) {
            return false;
        }
        if ($validate) {
            //we are trying to save.. need to verify that all is validated
            $this->validate();
            if ($this->hasInvalid()) {
                return false;
            }
        }
        return true;
    }
    
}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:

==> I2CE-4.1.3.tar.bz2: I2CE/modules/Forms/lib/I2CE_PageForm.php <==
<?php
/**
 * @copyright © 2007, 2008, 2009 Intrahealth International, Inc.
 * This File is part of I2CE
 *
 * I2CE is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/**
 * @package I2CE
 * @author Luke Duncan <lduncan@intrahealth.org>
 * @since v1.0.0
 * @version v2.0.0
 */
/**
 * Handles all page display and operations for forms.
 *  
 * This class extends the {@link I2CE_Page} class and adds some default methods and variables
 * for form templates that will interface with a {@link I2CE_Form} object to update the
 * database.  The {@link loadObjects()} method must be overwritten for all objects
 * extending this abstract class.
 * @package I2CE
 * @abstract
 * @access public
 * @see I2CE_Template
 */
abstract class I2CE_PageForm extends I2CE_PageFormBase {

    /**
     * Constant values for the type of object being set to be edited for this page: Primary
     * The primary object is the main object being edited by this page.  There can only be one.
     */
    const EDIT_PRIMARY = 1;
    /**
     * Constant values for the type of object being set to be edited for this page: Parent
     * The parent for the primary and secondary objects being edited by this page.  The can only be one.
     */
    const EDIT_PARENT = 2;
    /**
     * Constant values for the type of object being set to be edited for this page: Secondary
     * Any secondary objects being edited by this page.  They share the parent of the primary.  Multiple objects
     * can be added.
     */
    const EDIT_SECONDARY = 3;
    /**
     * Constant values for the type of object being set to be edited for this page: Children
     * Any child objects of the primary object will use this type.  Multiple objects can be added.
     */
    const EDIT_CHILD = 4;
        
    /**
     * The objects related to this page.  Using the PAGE_FORM_EDIT constants to determine the key for the 
     * array and the values will either be a {@link I2CE_Form} or an array of {@link I2CE_Form}s.
     * @var array
     */
    protected $objects;
    /**
     * The node id's of objects related to this page.
     * @var array
     */
    protected $node_ids;
        
    /**
     * Create a new instance of a form page.
     * 
     * This will call the constructor for all Page objects and then set up some additional
     * member variables for forms.
     * @param string $title The title for this page.
     * @param string $defaultHTMLFile The default HTML file for this page.
     * @param mixed $access The role required to access this page.
     * @param array $files The list of template files to load for this page.
     */
    public function __construct( $args,$request_remainder, $get = null,$post = null) {
        parent::__construct( $args,$request_remainder,$get,$post);
        $this->template->addHeaderLink('I2CE_ClassValues.js');
        $this->template->addHeaderLink('I2CE_SubmitButton.js');
        $this->node_ids = array();
        $this->objects = array( 
            self::EDIT_PRIMARY => false,
            self::EDIT_PARENT => false,
            self::EDIT_SECONDARY => array(),
            self::EDIT_CHILD => array(),
            );
    }
        
    /**
     * Add an object to the objects being edited or displayed by this page.
     * @param I2CE_Form $object  
     * @param integer $type The type of object being added from EDIT constants.
     * @param mixed $node_id  a tring which gives the node id for which this object applies to. also can be the DOMNode.   
     * Defaults to null meaning that it applies to the whole page
     *
     */
    protected final function setObject( $object, $type = self::EDIT_PRIMARY,$node_id = null ) {
        switch ( $type ) {
        case self::EDIT_PARENT :
        case self::EDIT_PRIMARY :
            $this->objects[$type] = $object;
            $this->node_ids[$type] = $node_id;
            break;
        case self::EDIT_SECONDARY :
        case self::EDIT_CHILD :
            $this->objects[$type][] = $object;
            $this->node_ids[$type][] = $node_id;
            break;
        }
    }
        
    /**
     * Return the primary object being edited.
     * @return I2CE_Form
     */
    protected function getPrimary() {
        return $this->objects[self::EDIT_PRIMARY];
    }
    /**
     * Return the parent object for this form.
     * @return I2CE_Form
     */
    protected function getParent() {
        return $this->objects[self::EDIT_PARENT];
    }
        
    /**
     * Set the I2CE_Form object in the page template.
     * 
     * This method will pass the edit object to the page template so that it can process all the form variables.
     */
    protected function setForm() {
        foreach( $this->objects as $type => $edit ) {
            if ( is_array( $edit ) ) {
                foreach( $edit as $i=> $obj ) {
                    if( $obj instanceof I2CE_Form ) {
                        if ($this->node_ids[$type][$i] !== null) {
                            $this->template->setForm( $obj, $this->node_ids[$type][$i] );
                        } else {
                            $this->template->setForm( $obj );
                        }
                    }
                }
            } elseif ( $edit instanceof I2CE_Form ) {
                if ($this->node_ids[$type] !== null) {
                    $this->template->setForm( $edit, $this->node_ids[$type] );
                } else {
                    $this->template->setForm( $edit );
                }
            }
        }
    }

    /**
     * Create and load any necessary objects for this form.
     * 
     * This method must be written for each class extending this class.
     */
    protected function loadObjects() {
        if ( $this->getParent() instanceof I2CE_Form ) {
            if ($this->getPrimary() instanceof I2CE_Form) {
                $this->getPrimary()->setParent( $this->getParent() );
            }
            foreach( $this->objects[self::EDIT_SECONDARY] as $secondary ) {
                $secondary->setParent( $this->getParent() );
            }
        }
        if ( $this->isPost() ) {
            if ($this->getPrimary() instanceof I2CE_Form) {
                $this->getPrimary()->load( $this->post);
            }
            foreach( $this->objects[self::EDIT_CHILD] as $child ) {                
                $child->load( $this->post ,  $child->getId() != '0');                
                $child->setParent( $this->getPrimary(), true );
            }
            foreach( $this->objects[self::EDIT_SECONDARY] as $secondary ) {
                $secondary->load( $this->post ,  $secondary->getId()!= '0');
            }
        } elseif ( $this->getPrimary() instanceof I2CE_Form ) {
            if ( $this->getPrimary()->getId() != '0' ) {
                $this->getPrimary()->populate();
                foreach( $this->objects[self::EDIT_CHILD] as $child ) {
                    $child->setParent( $this->getPrimary(), true );
                    $child->populate();
                }
                // Nothing needs to be done for secondary objects here because 2 objects can't be edited on the same form,
                // only new objects can be added to the database.
                $this->setEditing();
            }
        }
        return true;
    }
        

    /**
     * Run the validation methods for all the objects being edited.
     * 
     * If this is a form submit then run the validation methods for the default object being edited.  The default method
     * calls the {@link I2CE_Form::validate() validate} method on the {@link $edit_obj} object.
     */
    protected function validate() {
        if ($this->checked_validation) {
            return;
        }
        if ( $this->isPost() ) {
            foreach( $this->objects as $type => $edit ) {
                if ( $type == self::EDIT_PARENT ) continue;
                if ( is_array( $edit ) ) {
                    foreach( $edit as $i=>$obj ) {
                        if( $obj instanceof I2CE_Form ) {
                            $obj->validate();
                        }
                    }
                } elseif ( $edit instanceof I2CE_Form ) {
                    $edit->validate();
                }
            }
        }
        $this->checked_validation = true;
    }
                
    /**
     * Save the objects to the database.
     * 
     * Save the default object being edited b
     * @global array
     */
    protected function save() {
        $this->getPrimary()->save( $this->user );
        foreach( $this->objects[ self::EDIT_SECONDARY] as $obj ) {
            if (!$obj->save( $this->user )) {
                I2CE::raiseError("Could not save secondary object" . get_class($obj));
                return false;
            }
        }
        foreach( $this->objects[ self::EDIT_CHILD] as $obj ) {
            if ( $obj->getId() == '0' ) {
                $obj->setParent( $this->getPrimary() );
            }
            if (!$obj->save( $this->user )) {
                I2CE::raiseError("Could not save child object" . get_class($obj));
                return false;
            }
        }
        return true;
    }
        
    
        
    /**
     *Checks to see if any of the forms on this page have invalid messages
     *@returns boolean
     */
    public function hasInvalid() {
        $invalid = false;        
        foreach (array(self::EDIT_PARENT,self::EDIT_PRIMARY) as $key) {
            if ( !$this->objects[$key]  instanceof I2CE_Form) {
                continue;
            }
            $invalid |= $this->objects[$key]->hasInvalid();
        }
        foreach (array(self::EDIT_SECONDARY,self::EDIT_CHILD) as $key) {
            if (!is_array($this->objects[$key])) {
                continue;
            }
            foreach ($this->objects[$key] as $form) {
                if (!$form instanceof I2CE_Form) {
                    continue;
                }
                $invalid |= $form->hasInvalid();
            }
        }
        return $invalid;
    }

    /**
     * Add the form_error template to the page if the template is marked as invalid.
     */
    public  function invalidMessage() {  
        $node = parent::invalidMessage();
        if ($node instanceof DOMNode) {
            foreach (array(self::EDIT_PARENT,self::EDIT_PRIMARY) as $key) {
                if ( !$this->objects[$key]  instanceof I2CE_Form) {
                    continue;
                }
                if ( $this->objects[$key]->hasInvalid()) {
                    foreach ($this->objects[$key] as $field) {
                        if (!$field instanceof I2CE_FormField ) {
                            continue;
                        }
                        if ($field->hasInvalid()) {
                            $node->appendChild($this->template->createElement('span',array('style'=>'display:none'),$field->getHTMLName()));
                        }

                    }
                }
            }
            foreach (array(self::EDIT_SECONDARY,self::EDIT_CHILD) as $key) {
                if (!is_array($this->objects[$key])) {
                    continue;
                }
                foreach ($this->objects[$key] as $form) {
                    if (!$form instanceof I2CE_Form) {
                        continue;
                    }
                    if ($form->hasInvalid()) {
                        foreach ($form as $field) {
                            if (!$field instanceof I2CE_FormField ) {
                                continue;
                            }
                            if ($field->hasInvalid()) {
                                $node->appendChild($this->template->createElement('span',array('style'=>'display:none'),$field->getHTMLName()));
                            }
                        }
                    }
                }
            }
        }
        return $node;
    }

    
}

# Local Variables:
# mode: php
# c-default-style: "bsd"
# indent-tabs-mode: nil
# c-basic-offset: 4
# End:
